from sqlalchemy import *
from sqlalchemy import orm
from sqlalchemy.orm import create_session, relation

import datetime

from invirt.database import record

# Compatibility hack from
# http://www.sqlalchemy.org/trac/wiki/UsageRecipes/SessionAwareMapper
from sqlalchemy.orm import mapper as sqla_mapper

def session_mapper(scoped_session):
    def mapper(cls, *arg, **kw):
        if cls.__init__ is object.__init__:
            def __init__(self, **kwargs):
                for key, value in kwargs.items():
                    setattr(self, key, value)
                scoped_session.add(self)
            cls.__init__ = __init__
        cls.query = scoped_session.query_property()
        return sqla_mapper(cls, *arg, **kw)
    return mapper


__all__ = ['meta',
           'session',
           'clear_cache',
           'machine_table',
           'machine_access_table',
           'nic_table',
           'disk_table',
           'types_table',
           'cdroms_table',
           'mirrors_table',
           'autoinstalls_table',
           'owners_table',
           'admins_table',
           'builds_table',
           'Machine',
           'MachineAccess',
           'NIC',
           'Disk',
           'Type',
           'CDROM',
           'Mirror',
           'Autoinstall',
           'Owner',
           'Admin',
           'Build',
           'or_',
           ]

meta = ThreadLocalMetaData()
session = orm.scoped_session(orm.sessionmaker(autocommit=True, autoflush=False))

machine_table = Table('machines', meta,
       Column('machine_id', Integer, primary_key=True, nullable=False),
       Column('name', String, nullable=False, unique=True),
       Column('description', String, nullable=False),
       Column('memory', Integer, nullable=False),
       Column('owner', String, nullable=False),
       Column('contact', String, nullable=False),
       Column('uuid', String, nullable=False),
       Column('administrator', String, nullable=True, default=None),
       Column('type_id', String, ForeignKey('types.type_id'), nullable=False),
       Column('autorestart', Boolean, nullable=False, default=False),
       Column('cpus', Integer, nullable=False, default=1),
       Column('adminable', Boolean, nullable=False, default=False))

nic_table = Table('nics', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id'), nullable=True),
       Column('mac_addr', String, nullable=False, primary_key=True),
       Column('nic_type', String, nullable=False, default="pcnet"),
       Column('ip', String, nullable=False, unique=True),
       Column('netmask', String, nullable=False),
       Column('gateway', String, nullable=False),
       Column('other_ip', String, nullable=True),
       Column('other_netmask', String, nullable=True),
       Column('other_gateway', String, nullable=True),
       Column('other_action', String, nullable=True),
       Column('hostname', String, nullable=True),
       Column('reusable', Boolean, nullable=False, default=True))

disk_table = Table('disks', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id'), nullable=False),
       Column('guest_device_name', String, nullable=False),
       Column('size', Integer, nullable=False),
       PrimaryKeyConstraint('machine_id', 'guest_device_name'))

types_table = Table('types', meta,
       Column('type_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('hvm', Boolean, nullable=False),
       Column('apic', Boolean, nullable=False),
       Column('acpi', Boolean, nullable=False),
       Column('pae', Boolean, nullable=False))

mirrors_table = Table('mirrors', meta,
       Column('mirror_id', String, primary_key=True, nullable=False),
       Column('uri_prefix', String, nullable=False))

cdroms_table = Table('cdroms', meta,
       Column('cdrom_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('mirror_id', String, ForeignKey('mirrors.mirror_id')),
       Column('uri_suffix', String))

autoinstalls_table = Table('autoinstalls', meta,
       Column('autoinstall_id', String, primary_key=True, nullable=False),
       Column('description', String, nullable=False),
       Column('type_id', String, ForeignKey('types.type_id'), nullable=False),
       Column('distribution', String, nullable=False),
       Column('mirror', String, nullable=False),
       Column('arch', String, nullable=False),
       Column('preseed', Boolean, nullable=False, default=False))

owners_table = Table('owners', meta,
       Column('owner_id', String, primary_key=True, nullable=False),
       Column('ram_quota_total', Integer, nullable=True, default=None),
       Column('ram_quota_single', Integer, nullable=True, default=None),
       Column('disk_quota_total', Integer, nullable=True, default=None),
       Column('disk_quota_single', Integer, nullable=True, default=None),
       Column('vms_quota_total', Integer, nullable=True, default=None),
       Column('vms_quota_active', Integer, nullable=True, default=None))

builds_table = Table('builds', meta,
       Column('build_id', Integer, primary_key=True, nullable=False),
       Column('package', String, nullable=False),
       Column('pocket', String, nullable=False),
       Column('commit', String, nullable=False),
       Column('version', String, nullable=True, default=None),
       Column('principal', String, nullable=False),
       Column('succeeded', Boolean, nullable=True),
       Column('failed_stage', String, nullable=True, default=None),
       Column('traceback', String, nullable=True, default=None),
       Column('inserted_at', DateTime, nullable=False, default=datetime.datetime.utcnow))

machine_access_table = Table('machine_access', meta,
       Column('machine_id', Integer, ForeignKey('machines.machine_id', ondelete='CASCADE'), nullable=False, index=True),
       Column('user', String, nullable=False, index=True),
       PrimaryKeyConstraint('machine_id', 'user'))

admins_table = Table('admins', meta,
       Column('user', String, nullable=False, index=True, primary_key=True))

class Machine(record.Record):
    _identity_field = 'name'

class MachineAccess(record.Record):
    pass

class NIC(record.Record):
    pass

class Disk(record.Record):
    pass

class Type(record.Record):
    _identity_field = 'type_id'

class Mirror(record.Record):
    _identity_field = 'mirror_id'

class CDROM(record.Record):
    _identity_field = 'cdrom_id'

class Autoinstall(record.Record):
    _identity_field = 'autoinstall_id'

class Admin(record.Record):
    _identity_field = 'user'

class Build(record.Record):
    _identity_field = 'build_id'

from invirt.database.owner import Owner
mapper = session_mapper(session)

mapper(Machine, machine_table,
      properties={'nics': relation(NIC, backref="machine"),
                  'disks': relation(Disk, backref="machine"),
                  'type': relation(Type),
                  'acl': relation(MachineAccess, backref="machine", passive_deletes=True, cascade="all, delete-orphan")});
mapper(MachineAccess, machine_access_table)
mapper(NIC, nic_table)
mapper(Disk, disk_table)
mapper(Type, types_table)
mapper(Mirror, mirrors_table)
mapper(CDROM, cdroms_table,
       properties={'mirror': relation(Mirror, backref="cdroms")})
mapper(Autoinstall, autoinstalls_table)
mapper(Owner, owners_table)
mapper(Admin, admins_table)
mapper(Build, builds_table)

def clear_cache():
    """Clear sqlalchemy's cache
    """

    session.expunge_all()
